/*
 * $Header: /tmp_mnt/nocol/src/tpmon/RCS/tpmon.c,v 1.3 1992/06/18 21:21:44 spencer Exp $
 *
 * tpmon.c -- contains the throughput() function.  See tpmon.h
 * for calling parameters.
 *
 * $Log: tpmon.c,v $
 * Revision 1.3  1992/06/18  21:21:44  spencer
 * Had forgotten to close the socket.
 *
 * Revision 1.2  1992/06/15  19:31:05  aggarwal
 * Made the include file stuff more portable.
 *
 * Revision 1.1  1992/06/12  21:02:49  aggarwal
 * Initial revision
 *
 */

/*  Copyright 1992 JvNCnet */

#include <sys/param.h>
#include <sys/socket.h>
#include <sys/errno.h>
#include <time.h>
#include <sys/time.h>

#include <netinet/in.h>

#include <stdio.h>
#include <netdb.h>
#ifndef NeXT
#  include <unistd.h>
#endif	/* NeXT */
#include <signal.h>
#include <fcntl.h>
#include <setjmp.h>

extern char *prognm;
extern int errno;

/*
 * setup_sockaddr -- given address (e.g. phoenix.princeton.edu) or an
 *	IP # (e.g. 128.112.120.1), fills in the struct sockaddr_in
 *	passed to it, with the port number requested
 *
 * returns 0 on success, -1 on error
 */
int
setup_sockaddr(addr, to, port)
  char *addr;
  struct sockaddr_in *to;
  short int port;
{
  struct hostent *hp;
  int rtn = 0;

  bzero((char*)to, sizeof(struct sockaddr_in));
  to->sin_family = PF_INET;
  to->sin_addr.s_addr = inet_addr(addr);
  if (to->sin_addr.s_addr == (u_int)-1) {
    hp = gethostbyname(addr);
    if (hp == NULL) {
      fprintf(stderr, "%s: unknown host: %s\n", prognm, addr);
      rtn = -1;
    } else {
      to->sin_family = hp->h_addrtype;
      bcopy(hp->h_addr, (caddr_t)&to->sin_addr, hp->h_length);
    }
  }
  to->sin_port = htons(port);
  return rtn;
}

/*
 * make_buffer -- creates a buffer of size nbytes and fills it with
 * 	the pattern passed to it, or random chars if pattern is NULL
 */
char *
make_buffer(nbytes, pattern)
  int nbytes;
  char *pattern;
{
  char *buf;
  int plen;
  int i;

  if ((buf = (char *)malloc(nbytes+1)) != NULL) {
    if (pattern == NULL) {
      plen = 0;
      srand(getpid() ^ time((time_t *)0));
    } else
      plen = strlen(pattern);
    for (i = 0; i < nbytes; i++) {
      if (pattern == NULL)
        buf[i] = rand() % 256;
      else
        buf[i] = pattern[i % plen];
    }
  }
  return buf;
}

jmp_buf env;	/* for SIGALRM handling */

/*
 * stoptesting -- hack to break out of while loop in throughput()
 * 	activated by SIGALRM
 */
void
stoptesting()
{
  signal(SIGALRM, SIG_IGN);
  signal(SIGINT, SIG_DFL);
  longjmp(env, 1);
}

/*
 * throughput -- parameters and usage are described in tp.h and tpmon.h
 */
double
throughput(addr, port, numbytes, blocksize, pattern, time, verbose)
  char *addr, *pattern;
  long numbytes;
  int blocksize, time, verbose;
  short int port;
{
  int i, s; 			/* socket fd */
  struct sockaddr_in sockad;	/* our destination */
  struct hostent *hp;
  char *buf,			/* data buffer */
    error[81];			/* 81 sounded like a nice round number */
  int written,			/* actual # bytes written by write() */
    bytessent,	 		/* bytes sent successfully */
    badwrites;			/* write() returned something != blocksize */
  struct timeval tbeg, tend;    /* for timing purposes */
  double bits, secs, tp;	/* for calculating times */

  if (verbose) {
    if (time) 
      printf("%s: (throughput) sending for %d seconds\n", prognm, time);
    else
      printf("%s: (throughput) sending %.1fk bytes\n", prognm,
        (float)numbytes/1024.0);
  }
  /* this lets us put the program name in perror() messages */
  sprintf(error, "%s: ", prognm);

  /* first fill up the buffer... */
  if ((buf = make_buffer(blocksize, pattern)) == NULL) {
    fprintf(stderr, "%s: out of memory in make_buffer\n", prognm);
    return (double)-1.0;
  }

  /* now set things up for the writing */
  bytessent = badwrites = 0;
  if ((s = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
    strcat(error, "socket");
    perror(error);
    return (double)-1.0;
  }
  if (setup_sockaddr(addr, &sockad, port) < 0)
    return (double)-1.0;
  if (verbose) {
    /* find out remote site's full name for printing out later */
    hp = gethostbyaddr((char *)&sockad.sin_addr, sizeof(sockad.sin_addr),
      AF_INET);
    if (hp == NULL) {
      strcat(error, "gethostbyaddr");
      perror(error);
      return (double)-1.0;
    }
    addr = hp->h_name;
    printf("Connecting to %s [%s], port %d...\n", inet_ntoa(&sockad.sin_addr),
      addr, ntohs(sockad.sin_port));
  }    
  if (connect(s, &sockad, sizeof(struct sockaddr_in)) < 0) {
    strcat(error, "connect");
    perror(error);
    return (double)-1.0;
  }
  if (fcntl(s, F_SETFL, fcntl(s, F_GETFL, 0) | FNDELAY) < 0) {
    strcat(error, "fcntl");
    perror(error);
    return (double)-1.0;
  }

  /* if timing, set up the SIGALRM and set keepgoing to 1 for the loop */
  signal(SIGINT, stoptesting);
  if (time) {
    signal(SIGALRM, stoptesting);
    alarm(time);
  }
    
  /* on your marks... get set... */
  if (!setjmp(env)) {
    gettimeofday(&tbeg, (struct timezone *)NULL);
    do {
      if ((written = write(s, buf, blocksize)) < 0) {
        if ((errno == EWOULDBLOCK) || (errno == EINTR))
          continue;
        else
          ++badwrites;
      } else
        bytessent += written;
    } while (time || (bytessent < numbytes));
  }

  /*
   * now ignore SIGALRM, just in case both time and numblocks were
   * nonzero and we wrote out 'numbytes' bytes before 'time' seconds
   * expired.  Also alarm(0) to cancel any pending alarms.
   */
  alarm(0);
  signal(SIGALRM, SIG_IGN);
  signal(SIGINT, SIG_DFL);

  /* ok, we're done, how long did it really take? */
  gettimeofday(&tend, (struct timezone *)NULL);
  bits = (double)bytessent * (double)8.0;
  secs = (double)(tend.tv_sec - tbeg.tv_sec) +
         (double)(tend.tv_usec - tbeg.tv_usec) / (double)1000000.0;
  tp = bits / secs;

  if (verbose) 
    printf("Successfully sent %.1fk in %4.3g seconds (%d bad writes)\n",
      (float)(bytessent) / 1024.0, secs, badwrites);
  if ((close(s) < 0) && verbose) {
    strcat(error, "couldn't close socket [");
    strcat(error, inet_ntoa(&sockad.sin_addr));
    strcat(error, "] (non-fatal)");
    perror(error);
  }
  return tp;
}
